/*
  AD9850.h - Library for interfacing AD9850 function generator.
  Created by Raivis Strogonovs, November 25, 2013.
  Released into the public domain.
*/
 
#include "Arduino.h"
#include "AD9850.h"
 
/* 	PIN4 = D0
	PIN3 = D1
	PIN2 = D2
	PIN1 = D1
	*/
 
AD9850::AD9850(int W_CLK, int FQ_UD, int DATA, int RESET)
{
    _W_CLK = W_CLK; 
	_FQ_UD = FQ_UD; 
	_DATA = DATA; 
	_RESET = RESET;   
 
    pinMode(_W_CLK, OUTPUT);  
    pinMode(_FQ_UD, OUTPUT);
    pinMode(_DATA, OUTPUT);
	pinMode(_RESET, OUTPUT);

}

void AD9850::oscInt(unsigned long y)
{
  
    long z = 0;
        
	
	int i;
        
	//Frequency 32-bit word
	for (i=0;i<32;i++){
	        digitalWrite(_DATA, (y>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}
	
	//control bit #1, control bit #2 and Power off, all to low
	digitalWrite(_DATA, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	
	//phase 5-bit word
	for (i=0;i<5;i++){
                digitalWrite(_DATA, (z>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}

	digitalWrite(_FQ_UD, HIGH);
	digitalWrite(_FQ_UD, LOW);

}

void AD9850::osc(double frequency, double phase){
  
    Serial.println("Changing frequency to:");
    if(frequency < 1000000)
    {
      Serial.print(frequency/1000.0);
      Serial.println(" kHz");
    }
    else
    {
      Serial.print(frequency/1000000.0);
      Serial.println(" MHz");
    }

	long y=(long)frequency*FREQ_FACTOR/XTAL_MHZ;
	while(phase>360) phase-=360;
	long z=phase/11.5;
        
//    Serial.println("Frequency word:");
//    Serial.println(y);
	
	int i;
        
	//Frequency 32-bit word
	for (i=0; i<32; i++)
        {
	        digitalWrite(_DATA, (y>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}
	
	//control bit #1, control bit #2 and Power off, all to low
	digitalWrite(_DATA, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	
	//phase 5-bit word
	for (i=0; i<5; i++)
        {
                digitalWrite(_DATA, (z>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}

	digitalWrite(_FQ_UD, HIGH);
	digitalWrite(_FQ_UD, LOW);
	
//	Serial.println("Done.");
}

void AD9850::init()
{
        digitalWrite(_W_CLK, LOW);
	digitalWrite(_FQ_UD, LOW);
	digitalWrite(_DATA, LOW);
	digitalWrite(_RESET, LOW);
	Serial.println("All the pins lowered.");
}



void AD9850::doReset()
{	
  
        digitalWrite(_RESET, LOW);
	digitalWrite(_RESET, HIGH);
	digitalWrite(_RESET, LOW);
	
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);

        
	
	digitalWrite(_FQ_UD, LOW);
	digitalWrite(_FQ_UD, HIGH);
	digitalWrite(_FQ_UD, LOW);
	
osc(0,0);
	Serial.println("reset done...");
	
}

void AD9850::testFreq()
{
	unsigned long freq =  34359; /*Should be 1 kHz*/
	unsigned short phase = 0; /*No phase shift*/
	
	for(int i=0; i<32; i++)
	{
		digitalWrite(_DATA, ( freq >> i ) & 0x01 );
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}
	
	/*Clock out control bits and power down bit*/
	digitalWrite(_DATA, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	
	for(int i=0; i<5; i++)
	{
		digitalWrite(_DATA, ( phase >> i ) & 0x01 );
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}


	digitalWrite(_FQ_UD, HIGH);
        digitalWrite(_FQ_UD, LOW);
        
	Serial.println("Test frequency set...");
}
