﻿#ifndef F_CPU
#define F_CPU 32000000UL // Orangutans run at 20 MHz
#endif //!F_CPU

#include <avr/io.h>
#include "AD9850.h"

AD9850::AD9850(uint8_t W_CLK, uint8_t FQ_UD, uint8_t DATA, uint8_t RESET)
{
	this->_W_CLK = W_CLK;
	this->_FQ_UD = FQ_UD;
	this->_DATA = DATA;
	this->_RESET = RESET;	
}

void AD9850::init()
{
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_FQ_UD, LOW);
	digitalWrite(_DATA, LOW);
	digitalWrite(_RESET, LOW);
}

void AD9850::doReset()
{
	digitalWrite(_RESET, LOW);
	digitalWrite(_RESET, HIGH);
	digitalWrite(_RESET, LOW);
	
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
		
	digitalWrite(_FQ_UD, LOW);
	digitalWrite(_FQ_UD, HIGH);
	digitalWrite(_FQ_UD, LOW);
	
	osc(0,0);
	
	//digitalWrite(4, LOW);
}

void AD9850::osc(double frequency, double phase)
{
	long y=(long)frequency*FREQ_FACTOR/XTAL_MHZ;
	while(phase>360)
	phase-=360;
	long z=phase/11.5;
	
	//    Serial.println("Frequency word:");
	//    Serial.println(y);
	
	int i;
	
	//Frequency 32-bit word
	for (i=0;i<32;i++){
		digitalWrite(_DATA, (y>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}
	
	//control bit #1, control bit #2 and Power off, all to low
	digitalWrite(_DATA, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	
	//phase 5-bit word
	for (i=0;i<5;i++){
		digitalWrite(_DATA, (z>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}

	digitalWrite(_FQ_UD, HIGH);
	digitalWrite(_FQ_UD, LOW);
	
	//digitalWrite(4, HIGH);
	
}

void AD9850::oscInt(unsigned long y)
{
	
	long z = 0;
	
	
	int i;
	
	//Frequency 32-bit word
	for (i=0;i<32;i++){
		digitalWrite(_DATA, (y>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}
	
	//control bit #1, control bit #2 and Power off, all to low
	digitalWrite(_DATA, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	digitalWrite(_W_CLK, HIGH);
	digitalWrite(_W_CLK, LOW);
	
	//phase 5-bit word
	for (i=0;i<5;i++){
		digitalWrite(_DATA, (z>>i) & 0x01);
		digitalWrite(_W_CLK, HIGH);
		digitalWrite(_W_CLK, LOW);
	}

	digitalWrite(_FQ_UD, HIGH);
	digitalWrite(_FQ_UD, LOW);
	
	//digitalWrite(5, HIGH);

}

void AD9850::digitalWrite(uint8_t pin, uint8_t state)
{
	if(state > 0)
		_PORT_N |= (1<<pin);
	else
		_PORT_N &=~(1<<pin);
}