/*
 * Kulaks_firmware.cpp
 *
 * Created: 2014.04.14. 18:19:53
 *  Author: Raivis
 */ 

#define F_CPU 32000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include "SerialC.h"
#include "AD9850.h"

//For function generator
#define W_CLK 0
#define FQ_UD 1
#define DATA 2
#define RESET 3

//For sweeping DEFAULT VALUES, kulaks pc software can change it
#define N 160  //How many frequencies

#define MIN_FREQ 25000
#define MAX_FREQ 310000
#define SWEEP_STEP (MAX_FREQ-MIN_FREQ)/N




extern "C"{
	FILE * uart_str;
}

//Global variables
SerialC serial;
AD9850 funcGenerator(W_CLK, FQ_UD, DATA, RESET);

uint32_t singleStep;
uint32_t maxFreq;
uint32_t initialFreq;
uint32_t frequency = 1000;
uint16_t res = N;

float *results;            //-Filtered result buffer
float *freq;            //-Filtered result buffer

bool sweep;


//Prototype functions
void setUp32MhzInternalOsc();
static int uart_putchar (char c, FILE *stream);
int uart_getchar(FILE *stream);
void setUpSweep(uint32_t min, uint32_t max, uint16_t nRes);
void doSweep();
void setUpADC();
int readADC();

int main(void)
{
	setUp32MhzInternalOsc();
	
	 //Overwrite standard stdout;
	 uart_str = fdevopen(uart_putchar, uart_getchar); //send , receive functions
	 stdout = stdin = uart_str;
	 
	 //Setup Function Generator
	PORTC_DIR |= ((1<<W_CLK) | (1<<FQ_UD) | (1<<DATA) | (1<<RESET)); //Set the direction for function generator pins
	//PORTC_DIR = 0xff;
	//PORTC_DIR |= ((1<<4) | (1<<5));
	
	//printf("PORTA_DIR: %d\n", PORTA_DIR);
	 funcGenerator.init();
	 funcGenerator.doReset();
	 

	 funcGenerator.osc(2000,0);
	 
	 setUpSweep(MIN_FREQ, MAX_FREQ, N);
	 
	 //Setup ADC
	 PORTA_OUT = 0x00;
	 setUpADC();
	 
	 sweep = false;
	 
	 printf("Ready!\n");
	 
	// PORTC_OUTSET = PIN4_bm;
    while(1)
    {		
		//funcGenerator.osc(2000,0);
		//_delay_ms(200);
		//PORTC_OUT = ~PORTC_OUT;
		//_delay_ms(200);
		if(serial.available())
		{
			char inChar = serial.usart_receiveByte();
			if (inChar == 't')
			{
				char str [80];
				int i;
				printf ("Enter your family name: ");
				scanf ("%79s",str);
				printf ("Enter your age: ");
				scanf ("%d",&i);
				printf ("Mr. %s , %d years old.\n",str,i);
				printf ("Enter a hexadecimal number: ");
				scanf ("%x",&i);
				printf ("You have entered %#x (%d).\n",i,i);
			}
			else if(inChar == 'n')
			{				
				frequency += (float)singleStep*XTAL_MHZ/FREQ_FACTOR;
				funcGenerator.osc(frequency, 0);
				printf("Frequency: %0.2f kHz\n", frequency/1000.0);
			}
			else if(inChar == 'b')
			{
				frequency -= (float)singleStep*XTAL_MHZ/FREQ_FACTOR;
				funcGenerator.osc(frequency, 0);
				printf("Frequency: %0.2f kHz\n", frequency/1000.0);
			}
			else if(inChar == 'r')
			{
				frequency = (float)initialFreq*XTAL_MHZ/FREQ_FACTOR;
				funcGenerator.osc(frequency, 0);
				printf("Frequency: %0.2f kHz\n", frequency/1000.0);
			}
			else if(inChar == 'c')
			{
				printf("New sweep [min,max,n]:");
				uint32_t min;
				uint32_t max;
				uint16_t _n;
				
				char line[100];
				uint16_t i = 0;
				
				
				//if( fgets(line,100,stdin) && sscanf(line,"%lu %lu %u", &min, &max, &_n)!=1 )
				//{
					//min = 0;
					//max = 0;
					//_n = 0;
				//}
				scanf(" %lu %lu %u", &min, &max, &_n);
				//scanf(" %lu",&min);
				
				printf("Received %lu %lu %u\n", min,max,_n);
				//printf("Received %s %d\n", line, i);
				setUpSweep(min, max, _n);
			}
			else if(inChar == 's') //Stop
			{
				sweep = false;
			}				
			else if(inChar == 'd') //Do sweep
			{
				sweep = true;
			}
			else if(inChar == 'q')
			{
				//RESETS the whole AVR
				CCP = CCP_IOREG_gc; //Trigger protection mechanism
				RST_CTRL = RST_SWRST_bm;		
			}
		}
		
		if(sweep == true)
			doSweep();
		
		
		
    }
}

void doSweep()
{
	uint16_t d;
	frequency = initialFreq;
	funcGenerator.oscInt(frequency);
	
	serial.sendChar('{');
	
	for(d = 0; d<res; d++)
	{
		uint16_t v = readADC();
		frequency = frequency + singleStep;
		funcGenerator.oscInt(frequency);
		
		results[d] = results[d]*0.5+(float)(v)*0.5; //filter results
		
		printf("%0.2f ", results[d]);
	}
	
	serial.sendChar('}');
	serial.sendChar('\n');	
}


void setUpADC()
{
	ADCA_CH0_CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc; //Configure ADC conversion as single ended
	ADCA_CH0_MUXCTRL = ADC_CH_MUXPOS_PIN1_gc; // Set ADC to pin1, or PA1, because PA0 = AREFA
	
	//Sample rate = 166666 samples.
	// 1000000/(0.5*(12+0)+0) = 166666
	// See page 351
	ADCA_CTRLB = ADC_CURRLIMIT_LOW_gc | ADC_RESOLUTION_12BIT_gc; //Low current limit, up to 225ksps, and 12 bit resolution
	//ADCA_CTRLB = ADC_FREERUN_bm;
	//ADCA_REFCTRL = ADC_REFSEL_INTVCC_gc; // Internal reference VCC/1.6
	ADCA_REFCTRL = ADC_REFSEL_AREFA_gc; // External reference AVCC - 0.6
	ADCA_PRESCALER = ADC_PRESCALER_DIV32_gc; // Prescaler div 4
	
	ADCA_CTRLA = ADC_ENABLE_bm; //Enable ADC
	
}

int readADC()
{
	ADCA_CTRLA |= ADC_START_bm;
	//ADCA_INTFLAGS = 0x01;
	while(!(ADCA_CH0_INTFLAGS & ADC_CH0IF_bm));
	return ADCA_CH0_RES;
}

void setUpSweep(uint32_t min, uint32_t max, uint16_t nRes)
{
	res = nRes;
	
	maxFreq = (uint32_t)max*FREQ_FACTOR/XTAL_MHZ;
	initialFreq = (uint32_t)min*FREQ_FACTOR/XTAL_MHZ;
	singleStep = (maxFreq-initialFreq) / res;
	
	printf ("REs %d %d", res, nRes);
	
	free(results);
	results = (float*) malloc(res+1);
	free(freq);
	freq = (float*) malloc(res+1);
	
	printf("\nSweep initialized:\nMin: %0.2f kHz\nMax: %0.2f kHz\nStep:%0.2f kHz\n\n",min/1000.0,max/1000.0,((max-min)/res)/1000.0);
}

void setUp32MhzInternalOsc()
{
	OSC_CTRL |= OSC_RC32MEN_bm; //Setup 32Mhz crystal
	
	while(!(OSC_STATUS & OSC_RC32MRDY_bm));
	
	CCP = CCP_IOREG_gc; //Trigger protection mechanism
	CLK_CTRL = CLK_SCLKSEL_RC32M_gc; //Enable internal  32Mhz crystal
	
	
}

static int uart_putchar (char c, FILE *stream)
{
	if (c == '\n')
	uart_putchar('\r', stream);
	
	serial.sendChar(c);
	
	return 0;
}

int uart_getchar(FILE *stream)
{
	while( !serial.available() ); //Interesting DRIF didn't work.
	char data = serial.read();
	if(data == '\r')
	data = '\n';
	uart_putchar(data, stream);
	return data;
}