﻿#ifndef F_CPU
#define F_CPU 32000000UL // Orangutans run at 20 MHz
#endif //!F_CPU

#include <avr/io.h>
#include <stdio.h>
#include "SerialC.h"

SerialC::SerialC(void)
{
	setUpSerial();
}

void SerialC::setUpSerial()
{
	//For the sake of example, I'll just REMAP the USART pins from PC3 and PC2 to PC7 and PC6
	PORTC_REMAP |= 0x16; //See page 152 in datasheet, remaps the USART0
	
	PORTC_OUTSET = PIN7_bm; //Let's make PC7 as TX
	PORTC_DIRSET = PIN7_bm; //TX pin as output
	
	PORTC_OUTCLR = PIN6_bm;
	PORTC_DIRCLR = PIN6_bm; //PC6 as RX
	
	
	// Baud rate selection
	// BSEL = (32000000 / (2^0 * 8*115200) -1 = 34.7222 -> BSCALE = 0
	// FBAUD = ( (32000000)/(2^0*8(34+1)) = 114285.71 -> it's alright
	
	USARTC0_BAUDCTRLB = 0; //Just to be sure that BSCALE is 0
	USARTC0_BAUDCTRLA = 0x22; // 207
	
	
	//Disable interrupts
	USARTC0_CTRLA = 0;
	//8 data bits, no parity and 1 stop bit
	//USARTC0_CTRLC = USART_CMODE0_bm | USART_PMODE0_bm | USART_CHSIZE_8BIT_gc;
	USARTC0_CTRLC = USART_CHSIZE_8BIT_gc;
	
	//Enable receive and transmit
	USARTC0_CTRLB = USART_TXEN_bm | USART_CLK2X_bm | USART_RXEN_bm; // And enable high speed mode
	
}

void SerialC::sendChar(char c)
{
	
	while( !(USARTC0_STATUS & USART_DREIF_bm) ); //Wait until DATA buffer is empty
	
	USARTC0_DATA = c;
	
}

void SerialC::sendString(char *text)
{
	while(*text)
	{
		sendChar(*text++);
	}
}



char SerialC::usart_receiveByte()
{
	while( !(USARTC0_STATUS & USART_RXCIF_bm) ); //Interesting DRIF didn't work.
	return USARTC0_DATA;
}

uint8_t SerialC::available()
{
	
	return (USARTC0_STATUS & USART_RXCIF_bm);
}

char SerialC::read()
{
	return USARTC0_DATA;
}

