#include "learner.h"
#include "ui_learner.h"

Learner::Learner(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Learner)
{
    ui->setupUi(this);

    connect(ui->collectBtn,SIGNAL(clicked()), this, SLOT(startGathering()));
    connect(ui->cancelBtn, SIGNAL(clicked()), this, SLOT(close()));
    connect(ui->browseBtn, SIGNAL(clicked()), this, SLOT(openFileDialog()));
    connect(ui->learnBtn, SIGNAL(clicked()), this, SLOT(learn()));

    svmProcess =  new QProcess(this);

    ui->stopBtn->setVisible(false);

    connect(ui->stopBtn, SIGNAL(clicked()), svmProcess, SLOT(kill()));
}

void Learner::openFileDialog()
{
    QString fileName = QFileDialog::getSaveFileName(this,
            tr("Save SVM data"), "",
            tr("SVM data (*.svm);;All Files (*)"));

    if (fileName.isEmpty())
             return;
    else
    {
        //QFile file(fileName);
        ui->fileTxt->setText( fileName );
    }
}

void Learner::startGathering()
{
    if(ui->fileTxt->text().isEmpty())
    {
        QMessageBox::critical(this, tr("File not found"), tr("Please specify a file"));
        return;
    }
    if(ui->gestureName->text().isEmpty())
    {
        QMessageBox::critical(this, tr("Empty name"), tr("Please specify the name of gesture"));
        return;
    }

    //Send ignal to MainWIndow class, that I need Data
    emit gatherData(ui->spinTrain->value(), ui->spinId->value(), ui->fileTxt->text(), ui->gestureName->text());
    this->close();
}

void Learner::learn()
{
    //initiate process to construct svm model file
    QSettings settings("HKEY_CURRENT_USER\\Software\\Rastro\\Kulaks",
                       QSettings::NativeFormat);


    connect(svmProcess, SIGNAL(finished(int)), this, SLOT(enableButtons(int)));
    connect(svmProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(readProcess()));

    QStringList arguments;
    QString scriptPath = qApp->applicationDirPath() + "/libsvm/tools/";
    svmProcess->setWorkingDirectory(scriptPath);

    arguments << "easy.py" << ui->fileTxt->text() << ui->fileTxt->text();

    qDebug() << "Arguments: " << arguments;
//    qDebug() << "scriptPath";
    svmProcess->start( settings.value("python").toString() , arguments);



    ui->learnBtn->setEnabled(false);
    ui->cancelBtn->setEnabled(false);
    ui->collectBtn->setEnabled(false);

    ui->stopBtn->setVisible(true);
}

void Learner::readProcess()
{
    QByteArray bytes = svmProcess->readAllStandardOutput();
    qDebug() << bytes;
}

void Learner::enableButtons(int exitCode)
{
    ui->learnBtn->setEnabled(true);
    ui->cancelBtn->setEnabled(true);
    ui->collectBtn->setEnabled(true);

    ui->stopBtn->setVisible(false);

    qDebug() << "Process closed with: " << exitCode << svmProcess->readAllStandardError();

    if(exitCode > 0)
        QMessageBox::critical(this, "Something went wrong with QProcess", svmProcess->readAllStandardError());
}

Learner::~Learner()
{
    delete ui;
}
