#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QSerialPort>
#include <QMessageBox>
#include "qcustomplot.h"
#include "learner.h"
#include <QFile>
#include <QMessageBox>
#include "svm.h"
#include "settings.h"
#include <QSettings>
#include "serialterminal.h"
#include "playground/simulatekeyboard.h"
#include <QTimer>
#include <QThread>




namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    void writeToLogger(QString text);
    ~MainWindow();

private slots:
    void readData();
    void sendSerialData(QByteArray data);
    void openSerialPort();
    void closeSerialPort();
    void plot(QVector<double> values);
    void startGathering(int amount, int id, QString file, QString name);
    void openLearnerDialog();
    void doSomeMagic(QVector< double > samp);
    void openSettings();
    void openTerminal();
    void enableMagic(bool enable);
    void openSimulateKeyboard();
    void dataASecond();
    void sendDeviceSettings(int min, int max, int n);
    void aboutDialog();

private:
    Ui::MainWindow *ui;
    QSerialPort *serial;
    Learner *learner;
    SerialTerminal * terminal;
    Settings *settings;

    SimulateKeyboard *keyboardPlayground;

    QByteArray allData;
    QVector< QVector<double> > gatheredData;
    int numberOfDataStillNeeded;
    int numberOfDataNeeded;
    QString gestureName;
    QStringList gestureNameList;

    QTimer* dataASecondT;
    int collected;

    QString file;
    int id;

    int numberOfSamples;

    struct svm_model * model;

    double rmin[160];
    double rmax[160];

};

#endif // MAINWINDOW_H
