#include "settings.h"
#include "ui_settings.h"

Settings::Settings(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Settings)
{
    ui->setupUi(this);

    QList<QSerialPortInfo> ports = QSerialPortInfo::availablePorts();

    foreach(QSerialPortInfo port, ports)
        ui->serialPort->addItem(port.portName());

    foreach(qint32 baudrate, QSerialPortInfo::standardBaudRates())
        ui->baudrate->addItem(QString::number( baudrate ));

    connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(saveSettings()));
    connect(ui->pythonBtn, SIGNAL(clicked()), this, SLOT(pythonPathBtn()));
    connect(ui->gnuPlotBtn, SIGNAL(clicked()), this, SLOT(gnuplotPathBtn()));


    loadSettings();
}

void Settings::gnuplotPathBtn()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Select Gnuplot"),
                                                     "",
                                                     tr("Files (*.*)"));

    ui->gnuPlotPath->setText(fileName);
    saveSettings();
}

void Settings::pythonPathBtn()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Select Python"),
                                                     "",
                                                     tr("Files (*.*)"));

    ui->pythonPath->setText(fileName);
    saveSettings();


}

//SAves settings in the registry
void Settings::saveSettings()
{
    QSettings settings("HKEY_CURRENT_USER\\Software\\Rastro\\Kulaks",
                       QSettings::NativeFormat);
    settings.setValue("Serial/port", ui->serialPort->currentText());
    settings.setValue( "Serial/baudrate", ui->baudrate->currentText() );

    //Send new settings to the hardware
    emit deviceSettings(ui->minFreqSpinBox->value(), ui->maxFreqSpinBox->value(), ui->frequencyCountSpinbox->value());

    settings.setValue("Dev/min", ui->minFreqSpinBox->value());
    settings.setValue("Dev/max", ui->maxFreqSpinBox->value());
    settings.setValue("Dev/n", ui->frequencyCountSpinbox->value());

    settings.setValue("python", ui->pythonPath->text());
    settings.setValue("gnuplot", ui->gnuPlotPath->text());
}

//loads settings from the registry
void Settings::loadSettings()
{
    QSettings settings("HKEY_CURRENT_USER\\Software\\Rastro\\Kulaks",
                       QSettings::NativeFormat);

    ui->serialPort->setCurrentText( settings.value("Serial/port").toString() );
    ui->baudrate->setCurrentText( settings.value("Serial/baudrate").toString() );

    ui->minFreqSpinBox->setValue( settings.value("Dev/min").toInt() );
    ui->maxFreqSpinBox->setValue( settings.value("Dev/max").toInt() );
    ui->frequencyCountSpinbox->setValue( settings.value("Dev/n").toInt() );

    ui->pythonPath->setText( settings.value("python").toString() );
    ui->gnuPlotPath->setText( settings.value("gnuplot").toString() );
}

Settings::~Settings()
{
    delete ui;
}
